// Copyright 2019 Kina Wakasa
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'batch.dart';
import 'volatile_lokv.dart';

/// A in-memory implementation of [VolatileLoKV]
class InMemoryVolatileLoKV extends VolatileLoKV {
  Map<String, String> _db;

  InMemoryVolatileLoKV(String name) : super(name);

  @override
  Future<void> unsafeOpen() async {
    if (_db == null) {
      _db = Map<String, String>();
    }
  }

  @override
  Future<void> unsafeClose() async {}

  @override
  Future<void> unsafeDrop() async {
    _db = null;
  }

  @override
  Future<List<String>> unsafeRead(List<ReadOp> ops) async {
    var values = List<String>(ops.length);
    for (var i = 0; i < ops.length; ++i) {
      values[i] = _db[ops[i].key];
    }
    return values;
  }

  @override
  Future<void> unsafeWrite(List<WriteOp> ops) async {
    for (var op in ops) {
      if (op.value == null) {
        _db.remove(op.key);
      } else {
        _db[op.key] = op.value;
      }
    }
  }
}
